@@ -4,7 +4,7 @@ use warnings;
 package CPAN::Visitor;
 # ABSTRACT: Generic traversal of distributions in a CPAN repository
 
-our $VERSION = '0.004';
+our $VERSION = '0.005';
 
 use autodie;
 
@@ -20,7 +20,7 @@ use MooseX::Params::Validate 0.13;
 use namespace::autoclean 0.09 ;
 
 has 'cpan'  => ( is => 'ro', required => 1 );
-has 'quiet' => ( is => 'rw', default => 1 );
+has 'quiet' => ( is => 'ro', default => 0 );
 has 'stash' => ( is => 'ro', isa => 'HashRef',  default => sub { {} } );
 has 'files' => ( is => 'ro', isa => 'ArrayRef', default => sub { [] } );
 
@@ -168,7 +168,11 @@ sub _enter {
   my $curdir = Path::Class::dir()->absolute;
   my $target_dir = $job->{result}{extract} or return;
   if ( -d $target_dir ) {
-    chmod 0755, $target_dir unless -x $target_dir;
+    unless ( -x $target_dir ) {
+        warn "Directory '$target_dir' missing +x; trying to fix it\n"
+            unless $job->{quiet};
+        chmod 0755, $target_dir;
+    }
     chdir $target_dir;
   }
   else {
@@ -241,8 +245,7 @@ sub iterate {
   my $pm = Parallel::ForkManager->new( $params{jobs} > 1 ? $params{jobs} : 0 );
   for my $distfile ( @{ $self->files } ) {
     $pm->start and next;
-    eval { $self->_iterate($distfile, \%params) };
-    warn "Error visiting $distfile: $@\n" if $@ && ! $self->quiet;
+    $self->_iterate($distfile, \%params);
     $pm->finish;
   }
   $pm->wait_all_children;
@@ -251,23 +254,32 @@ sub iterate {
 
 sub _iterate {
   my ($self, $distfile, $params) = @_;
-  my $job = {
-    distfile  => $distfile,
-    distpath  => $self->_fullpath($distfile),
-    tempdir   => File::Temp->newdir(),
-    stash     => $self->stash,
-    quiet     => $self->quiet,
-    result    => {},
+  my $curdir = Path::Class::dir()->absolute;
+
+  # $job outside eval so that later chdir to original directory
+  # happens before $job is destroyed and any tempdirs deleted
+  my $job;
+  eval {
+    $job = {
+        distfile  => $distfile,
+        distpath  => $self->_fullpath($distfile),
+        tempdir   => File::Temp->newdir(),
+        stash     => $self->stash,
+        quiet     => $self->quiet,
+        result    => {},
+    };
+    $job->{result}{check} = $params->{check}->($job) or return;
+    $job->{result}{start} = $params->{start}->($job);
+    ACTION: {
+        $job->{result}{extract} = $params->{extract}->($job) or last ACTION;
+        $job->{result}{enter} = $params->{enter}->($job) or last ACTION;
+        $job->{result}{visit} = $params->{visit}->($job);
+        $job->{result}{leave} = $params->{leave}->($job);
+    }
+    $params->{finish}->($job);
   };
-  $job->{result}{check} = $params->{check}->($job) or return;
-  $job->{result}{start} = $params->{start}->($job);
-  ACTION: {
-    $job->{result}{extract} = $params->{extract}->($job) or last ACTION;
-    $job->{result}{enter} = $params->{enter}->($job) or last ACTION;
-    $job->{result}{visit} = $params->{visit}->($job);
-    $job->{result}{leave} = $params->{leave}->($job);
-  }
-  $params->{finish}->($job);
+  warn "Error visiting $distfile: $@\n" if $@ && ! $self->quiet;
+  chdir $curdir;
   return;
 }
 
@@ -295,7 +307,7 @@ CPAN::Visitor - Generic traversal of distributions in a CPAN repository
 
 =head1 VERSION
 
-version 0.004
+version 0.005
 
 =head1 SYNOPSIS
 
@@ -575,7 +587,7 @@ L<CPAN::Mini::Visit>
 =head2 Bugs / Feature Requests
 
 Please report any bugs or feature requests through the issue tracker
-at L<https://github.com/dagolden/cpan-visitor/issues>.
+at L<https://github.com/dagolden/CPAN-Visitor/issues>.
 You will be notified automatically of any progress on your issue.
 
 =head2 Source Code
@@ -583,9 +595,9 @@ You will be notified automatically of any progress on your issue.
 This is open source software.  The code repository is available for
 public review and contribution under the terms of the license.
 
-L<https://github.com/dagolden/cpan-visitor>
+L<https://github.com/dagolden/CPAN-Visitor>
 
-  git clone https://github.com/dagolden/cpan-visitor.git
+  git clone https://github.com/dagolden/CPAN-Visitor.git
 
 =head1 AUTHOR
 
